import 'package:flutter/material.dart';

class BaseDialog extends StatelessWidget {
  const BaseDialog({
    Key? key,
    required this.title,
    required this.onPressed,
    this.hiddenTitle = false,
    this.subTitle,
    this.cancelText = "取消",
    this.confirmText = "确定",
    this.height = 140,
  }) : super(key: key);

  final String title;
  final Function onPressed;
  final bool hiddenTitle;
  final String confirmText;
  final String? cancelText;
  final String? subTitle;
  final double height;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      resizeToAvoidBottomInset: false,
      backgroundColor: Colors.transparent,
      body: AnimatedContainer(
        alignment: Alignment.center,
        height: MediaQuery.of(context).size.height -
            MediaQuery.of(context).viewInsets.bottom,
        duration: const Duration(milliseconds: 120),
        curve: Curves.easeInCubic,
        child: Container(
            margin: const EdgeInsets.only(left: 65, right: 65),
            decoration: BoxDecoration(
              color: const Color(0xff2A272F),
              borderRadius: BorderRadius.circular(10.0),
            ),
            height: height,
            child: Column(
              children: <Widget>[
                Expanded(
                  child: Container(
                    alignment: Alignment.center,
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        hiddenTitle
                            ? Container()
                            : Padding(
                                padding:
                                    const EdgeInsets.only(left: 20, right: 20),
                                child: Text(
                                  title,
                                  textAlign: TextAlign.center,
                                  style: const TextStyle(
                                      fontSize: 15,
                                      color: Colors.white,
                                      fontWeight: FontWeight.bold),
                                ),
                              ),
                        hiddenTitle
                            ? Container()
                            : Container(
                                width: 8,
                              ),
                        subTitle == null
                            ? Container()
                            : Padding(
                                padding:
                                    const EdgeInsets.only(left: 12, right: 12),
                                child: Text(
                                  subTitle!,
                                  textAlign: TextAlign.center,
                                  style: const TextStyle(
                                      fontSize: 13, color: Color(0x70ffffff)),
                                ),
                              )
                      ],
                    ),
                  ),
                ),
                Container(
                  width: 375,
                  height: 1,
                  color: const Color(0x20ffffff),
                ),
                SizedBox(
                  height: 46,
                  child: Row(
                    children: [
                      Visibility(
                        visible: !TextUtil.isEmpty(cancelText),
                        child: Expanded(
                            child: GestureDetector(
                          onTap: () {
                            Navigator.of(context).pop();
                          },
                          child: Container(
                            color: Colors.transparent,
                            alignment: Alignment.center,
                            child: Text(
                              cancelText ?? "",
                              style: const TextStyle(
                                fontSize: 14,
                                color: Color(0x70ffffff),
                              ),
                            ),
                          ),
                        )),
                      ),
                      const SizedBox(
                        width: 1,
                        height: 46,
                        child: VerticalDivider(
                          color: Color(0x20ffffff),
                        ),
                      ),
                      Expanded(
                          child: GestureDetector(
                        onTap: () {
                          onPressed();
                        },
                        child: Container(
                          color: Colors.transparent,
                          alignment: Alignment.center,
                          child: Text(
                            confirmText,
                            style: const TextStyle(
                                fontSize: 14, color: Color(0x70ffffff)),
                          ),
                        ),
                      )),
                    ],
                  ),
                )
              ],
            )),
      ),
    );
  }
}

class TextUtil {
  static bool isEmpty(String? text) {
    return text == null || text.isEmpty;
  }
}

class DialogUtils {
  static show(
    BuildContext context, {
    required Widget widget,
    Color? barrierColor = const Color(0x44000000),
    bool useSafeArea = true,
    RouteSettings routeSettings = const RouteSettings(name: "showDialog"),
  }) {
    showDialog(
        context: context,
        barrierColor: barrierColor,
        useSafeArea: useSafeArea,
        useRootNavigator: true,
        routeSettings: routeSettings,
        builder: (BuildContext context) {
          return widget;
        });
  }
}
